// Keyhole International (Internal Script)
//
// nil, nil
//
// Turns debugging displays on. Non-player groups and waypoints are displayed
// on the map. Calling "khi_fnc_aiDebugOff" disables the display.

#define LEADER_ICON			"\a3\ui_f\data\map\vehicleicons\iconmanleader_ca.paa"
#define RIFLEMAN_ICON		"\a3\ui_f\data\map\vehicleicons\iconman_ca.paa"

if !(isNil "khi_aiDebugEnabled") exitWith { };

khi_aiDebugEnabled = true;

if (isNil "khi_aiDebugHostileRange") then
{
	khi_aiDebugHostileRange = 5000;
};

khi_aiDebugTerminate = false;

// An array of [_seeingUnit, _position, _accuracy, _cost] to display:
khi_aiDebugMarkers = [];

// An array of units to display:
khi_aiDebugUnits = [];

// An array of waypoints to display:
khi_aiDebugWaypoints = [];

// Hook the map draw handler:
khi_drawMarkers = 
{
	private ["_display", "_screenScale", "_dotSize"];

	_display = _this select 0;

	_screenScale = safeZoneW / (
		(_display ctrlMapWorldToScreen [1, 0, 0] select 0) -
		(_display ctrlMapWorldToScreen [0, 0, 0] select 0));

	_dotSize = _screenScale * 15 / 1920;

	{
		private ["_seeingUnit", "_position", "_accuracy", "_cost", "_seeingUnitPosition"];

		_seeingUnit = _x select 0;
		_position = _x select 1;
		_accuracy = _x select 2;
		_cost = _x select 3;

		_seeingUnitPosition = getPos _seeingUnit;

		// Draw the circle:
		_display drawEllipse [
			_position,
			_accuracy,
			_accuracy, 
			0, 
			[1, 0, 0, 1],
			""];

		// Draw a line from the unit to the circle:
		private ["_seeingDirection"];

		_seeingDirection = vectorNormalized (_position vectorDiff _seeingUnitPosition);

		_display drawLine [
			_seeingUnitPosition vectorAdd (_seeingDirection vectorMultiply _dotSize),
			_position vectorDiff (_seeingDirection vectorMultiply _accuracy),
			[1, 0, 0, 0.5]];

		// Draw a crosshair:
		_display drawLine [
			_position vectorAdd [-_accuracy, 0, 0],
			_position vectorAdd [_accuracy, 0, 0],
			[1, 0, 0, 0.5]];

		_display drawLine [
			_position vectorAdd [0, -_accuracy, 0],
			_position vectorAdd [0, _accuracy, 0],
			[1, 0, 0, 0.5]];
	}
	forEach khi_aiDebugMarkers;

	{
		private ["_leader", "_destination"];

		_leader = _x select 0;
		_destination = _x select 1;

		// Draw a line from the unit to the destination:
		private ["_leaderPosition", "_leaderDirection"];

		_leaderPosition = getPos _leader;
		_leaderDirection = vectorNormalized (_destination vectorDiff _leaderPosition);

		_display drawLine [
			_leaderPosition vectorAdd (_leaderDirection vectorMultiply _dotSize),
			_destination,
			[0, 0, 0, 0.8]];

		// Draw a crosshair:
		_display drawLine [
			_destination vectorAdd [-_dotSize, -_dotSize, 0],
			_destination vectorAdd [_dotSize, _dotSize, 0],
			[0, 0, 0, 0.8]];

		_display drawLine [
			_destination vectorAdd [-_dotSize, _dotSize, 0],
			_destination vectorAdd [_dotSize, -_dotSize, 0],
			[0, 0, 0, 0.8]];
	}
	forEach khi_aiDebugWaypoints;

	{
		{
			if (alive _x) then
			{
				private ["_unit", "_isLeader", "_icon", "_colour"];

				_unit = _x;

				_isLeader = leader group _unit == _unit;
				_icon = if _isLeader then { LEADER_ICON } else { RIFLEMAN_ICON };

				_colour = switch (side _x) do {
					case west: { [0.106, 0.459, 0.733, 0.9] };
					case east: { [0.945, 0.118, 0.176, 0.9] };
					case resistance: { [0.331, 0.453, 0.104, 0.9] };
					default { [0.756, 0.439, 0.086, 0.9] };
				};

				// Draw the seeing unit:
				_display drawIcon [
					_icon,
					_colour,
					getPos _unit,
					24, 
					24,
					getDir _unit,
					if _isLeader then { behaviour _seeingUnit } else { "" },
					0];
			};
		}
		forEach _x;
	}
	forEach khi_aiDebugUnits;

};

[] spawn 
{
	disableSerialization;

	waitUntil { !isNull findDisplay 12 };

	private ["_map", "_mapHandler"];

	_map = ((findDisplay 12) displayCtrl 51);

	_mapHandler = _map ctrlAddEventHandler ["Draw", '_this call khi_drawMarkers'];

	private ["_updateMarkers", "_enableGroupMarkers"];

	// Update the "khi_aiDebugMarkers" array with the current:
	_updateMarkers = 
	{
		// Do the update atomically since this routine may be interrupted by
		// the scheduler to paint a map frame:
		private ["_newMarkers", "_leader"];

		_newMarkers = [];

		{
			_leader = leader _x;

			if ({ isPlayer _x } count units _x == 0) then
			{
				// Update the targets:
				private ["_targets", "_targetSide", "_target"];

				_targets = _leader nearTargets khi_aiDebugHostileRange;

				{
					_target = _x select 4;

					if (isPlayer _target) then
					{
						private ["_position", "_cost", "_accuracy"];

						_position = _x select 0;
						_cost = _x select 3;
						_accuracy = _x select 5;

						_newMarkers pushBack
							[_leader, _position, _accuracy, _cost];
					};
				} foreach _targets;
			};
		}
		foreach allGroups;

		khi_aiDebugMarkers = _newMarkers;
	};

	_updateUnits = 
	{
		private "_newUnits";

		_newUnits = [];

		{
			if ({ isPlayer _x } count units _x == 0) then
			{
				_newUnits pushBack units _x;
			};
		}
		foreach allGroups;

		khi_aiDebugUnits = _newUnits;
	};

	_updateWaypoints =
	{
		private ["_newWaypoints", "_leader", "_position"];

		_newWaypoints = [];

		{
			if ({ isPlayer _x } count units _x == 0) then
			{
				_leader = leader _x;
				_position = getWPPos [_x, currentWaypoint _x];

				if ((_position select 0) != 0 || (_position select 1) != 0) then
				{
					_newWaypoints pushBack [_leader, _position];
				};
			};
		}
		foreach allGroups;

		khi_aiDebugWaypoints = _newWaypoints;
	};

	while { !khi_aiDebugTerminate } do
	{
		sleep 0.5;

		call _updateMarkers;
		call _updateUnits;
		call _updateWaypoints;
	};

	// Clean up:
	_map ctrlRemoveEventHandler ["Draw", _mapHandler];

	khi_aiDebugMarkers = nil;
	khi_aiDebugWaypoints = nil;
	khi_aiDebugUnits = nil;

	khi_drawMarkers = nil;

	khi_aiDebugHostileRange = nil;

	khi_aiDebugTerminate = nil;
	khi_aiDebugEnabled = nil;
};

